/*  "15 puzzle", expanded to be SIZE-squared puzzle.
     Written by Claude Anderson for CSSE 290.
	 April 11, 2013
*/

"use strict";

// CONSTANTS
var BOARD_SIZE = 500;
var BORDER_SIZE = 3;
var FONT_SIZE = 50;

// GLOBAL VARS

var tileList = [];
var blankTile;
var spacePos;
var SIZE; // 4 for 16 puzzle, 5 for 25 puzzle, etc.
var NUM_SQUARES;
var SQUARE_SIZE;


window.onload = function() {
	document.getElementById("choose").onclick =
	 function() {
	 	var radios = document.getElementsByName("size");
	 	for (var i=0; i<radios.length; i++){
	 		if(radios[i].checked) {
	 			SIZE = parseInt(radios[i].value);
	 			break;
	 		}
	 	}
	 	// In Section 01 a student noted that  the above six lines can be replaced by one line
	 	// SIZE = document.querySelector("input[name=size]:checked").value;
	 	NUM_SQUARES = SIZE * SIZE;
	 	SQUARE_SIZE = parseInt(BOARD_SIZE - (SIZE - 1) * (BORDER_SIZE *3 + 1)) / SIZE;
	 	
	 	var form = document.getElementById("form");
	 	form.parentNode.removeChild(form);
	 	
	 	setupBoard();
	 }
}


// called when page first loads to create tiles and empty space 
var setupBoard = function(){
	var boardDiv = document.getElementById("board");
	board.style.width = BOARD_SIZE + "px";
	for (var i = 1; i<=NUM_SQUARES ; i++) {
		//console.log("creating square" + i);
		var newSpan = document.createElement("span");
		newSpan.className = "tile";  // so CSS will work.
		newSpan.id = i;
		newSpan.innerHTML = i + " ";
		newSpan.style.width = SQUARE_SIZE + "px";
		newSpan.style.height = SQUARE_SIZE + "px";
		newSpan.style.lineHeight = SQUARE_SIZE + "px"; //vertically center the text.
		newSpan.style.border = BORDER_SIZE + "px solid red";
		newSpan.style.fontSize = FONT_SIZE + "px";
		tileList.push(newSpan);
		boardDiv.appendChild(newSpan);
		newSpan.onclick = moveTile;
	}
	blankTile = tileList[NUM_SQUARES - 1];
	blankTile.innerHTML = "&nbsp;";
	blankTile.classList.add("space");
	spacePos = NUM_SQUARES - 1;
	
	// Shuffle the tiles
	var minSwaps = NUM_SQUARES;
	var maxSwaps = NUM_SQUARES * 20;
	var numSwaps = minSwaps + Math.random()*(maxSwaps - minSwaps);
	var swapCount = 0;
	while (swapCount < numSwaps) {
		var index = parseInt(Math.random() * NUM_SQUARES);
		swapCount += moveTile.call(tileList[index]);
	}

}

// If clicked tile is next to empty space, swap them (in DOM and in tileList)
var moveTile = function() {
	// console.log("moveTile");
	var pos = tileList.indexOf(this); // "this" refers to the tile itself
	var diff = Math.abs(pos - spacePos);
	if (diff == SIZE  || diff==1 && sameRow(pos, spacePos, SIZE)) {
		console.log("swapping " + pos + " and " + spacePos);
		swapDomElements(blankTile, this);
		swapArrayElements(tileList, pos, spacePos);
		spacePos = pos;
		return 1;
	}
	return 0;
};

var swapArrayElements = function(a, i, j) {
	var temp = a[i];
	a[i] = a[j];
	a[j] = temp;
}

var sameRow = function(i, j, rowSize){
	return rowNum(i, rowSize) == rowNum(j, rowSize);
};

var rowNum = function(i, rowSize) {
	return parseInt(i / rowSize);
};

// Exchange the locations of two elements in the DOM.  Assumes that neither is the parent of the other.	
var swapDomElements = function(obj1, obj2) {  // from http://stackoverflow.com/questions/10716986/swap-2-html-elements-and-preserve-event-listeners-on-them
    // create marker element and insert it where obj1 is
    var temp = document.createElement("div");
    obj1.parentNode.insertBefore(temp, obj1);

    // move obj1 to right before obj2
    obj2.parentNode.insertBefore(obj1, obj2);

    // move obj2 to right before where obj1 used to be
    temp.parentNode.insertBefore(obj2, temp);

    // remove temporary marker node
    temp.parentNode.removeChild(temp);
}
	


